/*  Katy Perdue
    Created June 14, 2003
    Last Modified June 25, 2003 */

:-ensure_loaded(parse).
:-ensure_loaded(class).
:-ensure_loaded(createScale).

/* probScale takes in a chord, a melody (list of note names, no durations
   or octaves), and returns a set of scales that the melody might be a 
   part of, with the most probable ones listed first.  The Count is the 
   number of chromatic tones in the melody (using that particular scale).
   probScale2 does the exact same thing, though it also returns the full
   list of what's chromatic and what's not.  */

probScale(Chord, Melody, Set):-
    setof([Count, Name], Rest^degrees(Chord, Melody, [_, Name|Rest], Count), 
	  Set).


probScale2(Chord, Melody, Set):-
    setof([Count, List], degrees(Chord, Melody, List, Count), Set).



/* degrees that allows a user to enter a melody and a chord, and outputs
   a list of degrees the melody is composed of, and a count of the number 
   of times the word "chromatic" appears in the list.  It has support functions
   findDegrees, findDegrees1, checkMemeber, and chromaticCount. */

degrees(Chord, Melody, [Rank, RecScale|DegreeList], Count):-
    parse(Chord, Dur, ChordClass, Accidental, Qual, ExtensionList),
    recScale(Qual, RecScale, Rank),
    getScale([ChordClass, Accidental, RecScale], Scale),
    letterClass(ChordClass, ChordNum),
    findDegrees(ChordNum, Scale, Melody, DegreeList),
    chromaticCount(DegreeList, Count).


findDegrees(_, _,[], []):-!.
findDegrees(ChordNum, Scale, [FirstNote|Rest], [Degree|Degrees]):-
    findDegrees1(ChordNum, Scale, FirstNote, Degree),
    findDegrees(ChordNum, Scale, Rest, Degrees).

findDegrees1(ChordNum, Scale, FirstNote, DegreeList):-
    class(FirstNote, NoteClass),
    letterClass(NoteClass, NoteNum),
    checkMember(FirstNote, Scale, ChordNum, NoteNum, DegreeList).


/* checkMemeber finds the degree of the scale a "FirstNote" is, relative to 
   the scale "Scale".  Notes that are not contained the the scale are 
   marked with the word "chromatic". */ 

checkMember(FirstNote, Scale,ChordNum, NoteNum, DegreeList):-
    member(FirstNote, Scale),
    getDegree(ChordNum, NoteNum, DegreeList), !.
checkMember(_, _,_, _, chromatic).


/* chromaticCount (with the help of chromCount) counts the number
   of times the word "chromatic" appears in a list.  */

chromaticCount(First, ChromEnd):-
    chromCount(First, 0, ChromEnd), !.
chromCount([], X, X).
chromCount([chromatic|Rest], A, N):-
    A1 is A + 1,
    chromCount(Rest, A1,N ).
chromCount([_|Rest], A,N ):-
    chromCount(Rest, A, N).


/* This is a shortened list of the Aebersold suggested scales. */

recScale('major', maj, 1).
recScale('major', aug, 2).
recScale('major', blues, 3).
recScale('major', dimH, 4).
recScale('major', pent, 5).
recScale('major', harmMin6Mode, 6).

recScale('dominant', dom, 1).
recScale('dominant', pent, 2).
recScale('dominant', bebopDom, 3).
recScale('dominant', blues, 4).

recScale('minor', dor, 1).
recScale('minor', bebopMin, 2).
recScale('minor', harmMin, 3).
recScale('minor', blues, 4).
recScale('minor', pentMin, 5).
recScale('minor', dimW, 6).

recScale('locrian or half diminished', bebopLoc, 1).
recScale('locrian or half diminished', loc,2 ).

recScale('diminished', dim8, 1).



/* getScale takes in note, accidental, and scale quality, and returns 
   the specified scale.  switch is a helper predicate.  */

getScale([Note, Accidental, Type], ReturnScale):-
    scale(Note, Accidental, Scale),
    switch([Type, Scale, ReturnScale]).

switch(X):-
    Goal=..X, call(Goal).



/*getDegree uses numerical comparison to find the degree of the scale
  that of the second input. */

getDegree(X,X, root).
getDegree(ChordNum, NoteNum, second):- next(ChordNum, NoteNum), !.
getDegree(ChordNum, NoteNum, third):- next(ChordNum, X), 
    next(X, NoteNum), !.
getDegree(ChordNum, NoteNum, fourth):- next(ChordNum, X), next(X, Third),
    next(Third, NoteNum), !.
getDegree(ChordNum, NoteNum, fifth):- next(ChordNum, X), next(X, Third),
    next(Third, Fourth), next(Fourth, NoteNum),!.
getDegree(ChordNum, NoteNum, sixth):- next(ChordNum, X), next(X, Third),
    next(Third, Fourth), next(Fourth, Fifth), next(Fifth, NoteNum),!.
getDegree(ChordNum, NoteNum, seventh):- next(ChordNum, X), next(X, Third),
    next(Third, Fourth), next(Fourth, Fifth), next(Fifth, Sixth), 
    next(Sixth, NoteNum).


/* next is a support function for getDegree.  It puts the numbers
   0 through 6 (coresponding to A through G) on a closed loop.
   The idea to represent notes this way was taken from the Prolog
   in music book. */

next(X, Y):- Y is X+1.
next(6,0).


/* The classic member function.*/
member(X, [X|_]).
member(X, [_|R]) :- member(X, R).


/* letterClass assigns a numerical value to each class. */

letterClass('C', 0).
letterClass('D', 1).
letterClass('E', 2).
letterClass('F', 3).
letterClass('G', 4).
letterClass('A', 5).
letterClass('B', 6).
